import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import controlP5.*; 
import processing.pdf.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class alphagraph_09 extends PApplet {

/*
 * PLEASE READ INFO TAB FOR FURTHER INFO
 * NOTES : Version _085 used for workshop METZ 25, 26, 27/01/2017
 */

///////////////// GLOBALS ///////////////////////////////


boolean SAVE_PDF = false;

ControlP5 INTERFACES;
Grid LINE_AGENTS;
Grid DOT_AGENTS;

///////////////// SETUP ///////////////////////////////
public void setup() {
  
  surface.setResizable(true);
  colorMode(HSB, 360, 100, 100);
  background(0, 0, 100);
  
  controlInit(); // initialise GUI (Contains variable instances too !)
  initPGraphics(); // initialise PGraphics ( FONTS )

  // create a grid & render with x, y & step values
  LINE_AGENTS = new Grid();
  LINE_AGENTS.renderGrid(XOFF, YOFF, STEPX, STEPY);
  DOT_AGENTS = new Grid();
  DOT_AGENTS.renderGrid(XOFF, YOFF, STEPXDOT, STEPYDOT);

}

//////////////////// DRAW ///////////////////////////////
public void draw() {
  if (SAVE_PDF) beginRecord(PDF, "exportPDF_###.pdf");
  colorMode(HSB, 360, 100, 100);
  background(0, 0, 100);
  updatePGraphics(); // update PGraphics ( TEXT )

  if (isInvert) {
    if (SHOWCIRCLES) {
      DOT_AGENTS.drawCircleAgents();
    }

    if (SHOWLINES) {
      LINE_AGENTS.drawLineAgents();
    }
  } else {

    if (SHOWLINES) {
      LINE_AGENTS.drawLineAgents();
    }
    if (SHOWCIRCLES) {
      DOT_AGENTS.drawCircleAgents();

    }
  }

  if (SAVE_PDF) {
    endRecord();
    SAVE_PDF = false;
  }

  updateGrid(XOFF, YOFF, STEPX, STEPY, STEPXDOT, STEPYDOT);
}

///////////////////////////////////// FIN
/*
 * The Agent class draws the graphical elements - point & line
 * as well as controls various motions/animations
 *
 */

////////////////////////////////////// CLASS 
class Agent {
  float x, y;
  float dia, len, strokeW, angle;
  int c;
  
  // SUPER PERLIN
  int noiseRes = 5;
  float noiseVal;
  float noiseScale = 0.001f;
  float noiseCount = 0.0f;
  float noiseSpeed = 0.002f;
  float xv, yv;

  float speed = 900;
  float xCount, xSpeed;
  float yCount, ySpeed;
  float theta;


  ///////////////////////////////////// CONSTRUCTOR 01
  Agent(float x, float y) {
    this.x = x;
    this.y = y;
    dia = DIA;
    len = LEN;
    strokeW = STROKEW;
    angle = 0;
  }

  //////////////////////////// METHODS

  // General method that brings all together
  public void renderAgents() {

    if (NOISE_01) {
      noise_01(KEN_P_AMM, KEN_P_FACT*KEN_F_AMM);
    }

    if (NOISE_02) {
      noise_02(KEN_P_AMM, KEN_P_FACT*KEN_F_AMM); // original
    }

    if (ANIME_LINES) {
      noise_03(KEN_P_AMM, KEN_P_FACT*KEN_F_AMM);
    }


    if (ANIMATE) {
      float nAMM = map(mouseX, 0, width, 1, 73);
      float nF = map(mouseY, 0, height, 0.0001f, 0.009f);
      float nFAuto = cos(frameCount*(nF*0.1f)) * 0.05f;
      noise_01(nAMM, nFAuto);
    }
  }

  //////////////////////////////////////////////////////////// MOTION METHODS

  // 2D noise animation of circle diameters (WITH MOTION)
  public void noise_01(float nAmm, float nFac) {
    float noiseAlgo1 = noise((x+frameCount) * nFac, (y+frameCount) * nFac) * nAmm;
    dia = noiseAlgo1 + DIA;
  }

  // 2D noise animation of circle diameters (WITHOUT MOTION)
  public void noise_02(float nAmm, float nFac) {
    float noiseAlgo2 = noise((y+nFac) * nFac) * nAmm;
    dia = noiseAlgo2 + DIA;
    //strokeW = noiseAlgo2 + STROKE_W/5; // add stroke ?
  }

  // 2D noise animation of line stroke weight (WITH MOTION)
  public void noise_03(float nAmm, float nFac) {
    float noiseAlgo3 = noise((x+nFac) * nFac, (y+frameCount) *nFac) * nAmm;
    strokeW = noiseAlgo3 + STROKEW/5;
  }


  // Mouse click motion
  public void motion(float nAmm) {
    angle += noise(x*0.005f, y*0.005f) * nAmm;
    //angle += noise(x*0.005, y*0.005) * nAmm;
    x += cos(angle*0.05f) * 3;
    y += sin(angle*0.05f) * 3;
  }

  ////////////////////////////////////////// GRAPHIC
  public void displayDots() {
    c = g.get(PApplet.parseInt(x), PApplet.parseInt(y));
    if ((c >= color(0)) && (c <= color(30))) {

      if (isStrokeC) {
        noFill();
        strokeWeight(STROKEC);
        stroke( HUE, SAT, BRIGHTNESS );
      } else {
        noStroke();
        fill(HUE, SAT, BRIGHTNESS);
      }
     // noStroke();
     // fill(HUE, SAT, BRIGHTNESS);
      pushMatrix();
      translate(x, y);
      rotate( angle+THETA_CIRCLES );
      if (SYMMETRIC) {
        // Experimental form : See function below
        //circularForm(0,0,10,30,DIA+dia);
        ellipse(0, 0, DIA+dia, DIA+dia);
      } else {
        ellipse(0, 0, DIAX+dia, DIAY+dia); // PLUS dia Here ?
      }
      popMatrix();
    }
  }

  public void displayLines() {  
    xSpeed = ((width/2) - mouseX)/10.0f;
    ySpeed = ((height/2) - mouseY)/10.0f;

    xCount += xSpeed;
    yCount += ySpeed;  
    c = g.get(PApplet.parseInt(x), PApplet.parseInt(y));
    if ((c >= color(0)) && (c <= color(30))) {
      noFill();
      strokeWeight(strokeW);
      strokeCap(SQUARE);
      stroke(HUE2, SAT2, BRIGHTNESS2);

      pushMatrix();
      translate(x, y);

      // PERLIN NOISE ANGLE
      if (NOISE_THETA_LINE) {
        float snx = sin(angle * 0.005f) * 30; // Smoother
        float sny = sin(angle * 0.005f) * 30; // Smoother

        float pr = noise(x * PL_SCALE + snx, y * PL_SCALE + sny) * PL_AMM;
        angle  = (-(pr * PI ))/180.0f;

        rotate( radians( pr ) + angle );
        xv = cos(angle) * speed;
        xv = cos(angle) * speed;

        rotate(angle);
        float lenX = xv / len;
        float lenY = yv / len;
        line (-lenX, -lenY, 0, 0);
      } else {
        //TL = ceil(THETA_LINE);
        //rotate(radians(TL));
        rotate(radians(ceil(THETALINE)));
        line (-len, -len, len, len);
      }

      popMatrix();
    }
  }
  // Another graphic form
  public void circularForm(float _x, float _y, float _radius, float _step, float _dia) {
    noStroke();
    fill(HUE, SAT, BRIGHTNESS);
    float radius = _radius; // circle radius
    float angleStep = _step; // angle step = number of repetitions

    pushMatrix();
    translate(_x, _y);
    for (float deg =0; deg<=360; deg+=angleStep) {
      // Calculate our x & y positions on the circle
      float xPos = cos(radians(deg))*radius; 
      float yPos = sin(radians(deg))*radius;
      // Draw shape
      ellipse(xPos, yPos, _dia, _dia);
    }
    popMatrix();
  }



  // !!! NOT IMPLEMENTED YET !!! 16/10/2015
  public void superPerlin() {
    if (x%noiseRes == 0 && y%noiseRes == 0) {
      noiseVal=noise((x - xCount)*noiseScale, (y - yCount)*noiseScale, noiseCount);
      float g = noiseVal*720.0f;

      theta = (-(g * PI))/180.0f;
      xv = cos(theta)  * speed;
      yv = sin(theta)  * speed;
    }
  }
}
/*
 * A messy set of functions for key interaction and initial data setup
 */

PGraphics g;
String txt = "hotdogs";
int fontIndex;
PFont f;
String[] s;
//////////////////// GRAPHIC FUNCTIONS //////////////////////////

public void initPGraphics() {
  g = createGraphics(width, height, JAVA2D);
  s = PFont.list();
  //printArray(s);
  THETAFINE = 0.5f;

  g.beginDraw();
  g.colorMode(HSB, 360, 100, 100);
  fontIndex = 0;
  //  PFont f = createFont("FiraSans-ExtraBold", 100);
  f = createFont(s[fontIndex], FONT_SIZE);
  g.textFont(f, FONT_SIZE);
  g.textSize( FONT_SIZE );
  g.endDraw();
}

public void updatePGraphics() {
  g.beginDraw();
  //g.colorMode(HSB,360,100,100);
  g.background(0, 0, 255);
  f = createFont(s[fontIndex], FONT_SIZE);
  g.textFont(f, FONT_SIZE);
  g.textSize( FONT_SIZE );
  g.textAlign(CENTER, CENTER);
  g.fill(0);
  g.text(txt, g.width/2, g.height/2);
  g.endDraw();

  // DEBUG
  //image(g, 0, 0);
}



//////////////////// INTERACTION : KEYS //////////////////////////
public void keyPressed() {

  if(INTERFACES.isMouseOver()==false)
  if (keyCode == BACKSPACE) {
    if (txt.length() > 0) {
      txt = txt.substring(0, txt.length()-1);
    }
  } else if (keyCode == DELETE) {
    txt = "";
  } else if (keyCode != ENTER && key != '-' && keyCode != ALT && keyCode != RIGHT && keyCode != LEFT) {
    fill(255);
    txt = txt + key;
  }

  if (key == '-') {
    saveFrame("StringAgents_###.png");
  }
}

////////////////////// KEYS
public void keyReleased() {
  if (keyCode == ENTER) {
    DOT_AGENTS.AgentsCollection.clear();
    DOT_AGENTS.renderGrid(XOFF, YOFF, STEPX, STEPY);
    LINE_AGENTS.AgentsCollection.clear();
    LINE_AGENTS.renderGrid(XOFF, YOFF, STEPX, STEPY);
  }

  // Added angle rotation fine tuning.
  if (keyCode == RIGHT) {
    THETALINE += THETAFINE;
    float temp =  INTERFACES.getController("THETALINE").getValue();
    float newVal = temp + THETAFINE;
    INTERFACES.getController("THETALINE").setValue(newVal);
  }
  if (keyCode == LEFT) {
    THETALINE -= THETAFINE;
    float temp =  INTERFACES.getController("THETALINE").getValue();
    float newVal = temp - THETAFINE;
    INTERFACES.getController("THETALINE").setValue(newVal);
  }
}
/*
 * The Grid class generates a grid of Agents
 * It also controls the display of these Agents
 * as points &/or lines.
 *
 */
////////////////////////////////

class Grid {
  ArrayList <Agent> AgentsCollection;

  Grid() {
    AgentsCollection = new ArrayList<Agent>();
  }


  //////////////////////////////////////////////// M\u00c9THODES
  public void renderGrid(float xOff, float yOff, float interX, float interY) {
    AgentsCollection.clear(); // IMPORTANT ! 

    for (float y=10; y<height-10; y+=interY) {
      for (float x=10; x<width-10; x+=interX) {

        // Initialise each object with the constructor
        Agent myAgent = new Agent(x+xOff, y+yOff);

        // Add each object to the ArrayList
        AgentsCollection.add(myAgent);
      }
    }
  }

  // DRAW AGENTS ON GRID AS DOTS
  public void drawCircleAgents() {    
    for (Agent ag : AgentsCollection) {

      if (MOUSE_ANIME) {
        if (mousePressed) {
          float nAMM_Motion = map(mouseY, 0, height, 1, 500);
          ag.motion( nAMM_Motion );
        }
      } 

      ag.renderAgents();
      ag.displayDots();
    }
  }

  // DRAW AGENTS ON GRID AS LINES
  public void drawLineAgents() {
    for (Agent ag : AgentsCollection) {
      ag.dia = DIA;
      ag.len = LEN;
      ag.strokeW = STROKEW;
      ag.angle = THETALINE; 

      if (MOUSE_ANIME) {
        if (mousePressed) {
          float nAMM_Motion = map(mouseY, 0, height, 1, 500);
          ag.motion( nAMM_Motion );
        }
      } 
      ag.renderAgents();
      ag.displayLines();
    }
  }
}
///////////////////////////////////////////////////////// FIN
/*
 :::::::::::::::::::::::::::::::
 ------------------------------
 PROJECT : LETTERS & FORM 2015  
 ------------------------------
 ::::::::::::::::::::::::::::::
 
 DEV 26/01/2017
 Updated for latest version of ControlP5
 Sketch : alphaGraph 
 
 - This software helps in generating ideas for letter forms ;\u2013)
 
 - GIHUB REPO : https://github.com/FreeArtBureau/alphaGraph
 - WORKSHOPS : http://workshop-lettrine-01.esad-amiens.fr/
 
 NOTES TO SELF :
 
 - Write a clear set of instructions.
 
 *******************************************************
 * LETTERS & FORM is a series of software that explores 
 graphical possibilites with letter forms.
 *******************************************************
 Written by MW2015
 www.freeartbureau.org
 
 INSTRUCTIONS
 ------------
 - Type text and play with various parameters.
 - MousePressed when "MOUSE" button has been activated deforms text. 
 The deform function is wired to mouse x & y position creating different effects accordingly.
 - ENTER > Clear screen & create a new grid.
 - exportPDF button exports to PDF.
 - SAVE & LOAD buttons enable one to save and load presets as separate files.
 
 ///////////////////////////// END
 
 
 * Copyright (c) 2015 Mark Webster
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * http://www.gnu.org/licenses/gpl.html
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 
 */
/*
 * A very sloppy collection of variables
 * along with all GUI objects & methods.
 *
 */

//////////////////////// INTERFACE VARIABLES
float STROKEW, STROKEC;
float DIA, DIAX, DIAY, LEN;
int THETALINE;
// variable for fine tuning of angles with right & left keys
float THETAFINE;

float THETA_CIRCLES;
float KEN_P_AMM, KEN_P_FACT, KEN_F_AMM, PL_SCALE, PL_AMM;
int HUE, SAT, BRIGHTNESS, HUE2, SAT2, BRIGHTNESS2;

float STEPX, STEPY, XOFF, YOFF, STEPXDOT, STEPYDOT;
boolean SHOWLINES, NOISE_01, NOISE_02, ANIMATE, ANIME_LINES,
  MOUSE_ANIME, NOISE_THETA_LINE, SUPERPERLIN_LINE;
boolean SHOWCIRCLES, SYMMETRIC, isStrokeC, isInvert;
boolean SAVEPRESET, LOADPRESET;
String getFont;

float FONT_SIZE;

Toggle mt1;
Toggle mt2;
Textfield myTextfield;
ScrollableList d1;

//////////////////////////////////////////

public void controlInit() {
  PFont p = loadFont("FiraMono-Regular-11.vlw");
  textFont(p, 11);
  INTERFACES  = new ControlP5(this, p);
  //fontIndex = 1;

  // Custom scrollable list
  String[] fontList = PFont.list();
  d1 = INTERFACES.addScrollableList("FontsList")
    .setPosition(800, 5)
    .setWidth(180)
    .setBarHeight(20)
    .setItemHeight(25)
    .open()
    .addItems(fontList);

  //////////////////////////////////// Add GROUP 1
  Group g1  =  INTERFACES.addGroup("g1")
    .setPosition(10, 20)
    .setWidth(180)
    .activateEvent(true)
    .setBackgroundColor(color( 0xffCED2DB ))  //#CED2DB, #C9BFBF
    .setBackgroundHeight(245)
    .setLabel("FORM")
    .setBarHeight(14);

  ///////////////////////////////////////////////////////////////////// GROUP 1 SETTINGS GEOMETRY
  // LINES
  Slider ls1 = INTERFACES.addSlider("LEN").setPosition(10, 20)
    .setRange(0.0f, 60.0f).setValue(50.0f).setHeight(20).setWidth(120)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);


  Numberbox nb1 = INTERFACES.addNumberbox("STROKEW").setPosition(10, 45)
    .setRange(0.0f, 60.0f).setValue(2.0f)
      .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
        .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);
          makeEditable( nb1 );


  Numberbox nb2 = INTERFACES.addNumberbox("THETALINE").setPosition(85, 45)
    .setRange(ceil(0), ceil(360)).setValue(0).setCaptionLabel("ROTATE")
    //.setNumberOfTickMarks(4)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);
  makeEditable( nb2 );


  Toggle lt1 = INTERFACES.addToggle("SHOWLINES").setPosition(10, 90)
    .setSize(15, 15).setValue(true).setCaptionLabel("LINE")
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);


  Toggle lt2 = INTERFACES.addToggle("SHOWCIRCLES").setPosition(60, 90)
    .setSize(15, 15).setValue(true).setCaptionLabel("POINT")
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);


  Toggle lt3 = INTERFACES.addToggle("SYMMETRIC").setPosition(115, 90)
    .setSize(15, 15).setValue(true)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);


  Toggle lt4 = INTERFACES.addToggle("isStrokeC").setPosition(90, 200)
    .setSize(15, 15).setValue(false).setCaptionLabel("stroke")
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);


  Toggle t5 = INTERFACES.addToggle("isInvert").setPosition(135, 200)
    .setSize(15, 15).setValue(false).setCaptionLabel("INVERT")
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);


  // CIRCLES
  Slider ls4 = INTERFACES.addSlider("DIA").setPosition(10, 130)
    .setRange(0.0f, 40.0f).setValue(5.0f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);


  Slider ls5 = INTERFACES.addSlider("DIAX").setPosition(10, 145)
    .setRange(0.0f, 40.0f).setValue(2.0f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);

  Slider ls6 = INTERFACES.addSlider("DIAY").setPosition(10, 160)
    .setRange(0.0f, 40.0f).setValue(2.0f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);

  Slider ls7 = INTERFACES.addSlider("THETA_CIRCLES").setPosition(10, 175)
    .setRange(0.0f, TWO_PI).setValue(2.0f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);
  ls7.setLabel("ROTATE");

  Numberbox nb3 = INTERFACES.addNumberbox("STROKEC").setPosition(10, 200)
    .setRange(0.1f, 50.0f).setValue(0.5f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g1);
  nb3.setLabel("THICKNESS");
  makeEditable( nb3 );


  ///////////////////////////////////////////////////////////////////// GROUP 2 COLOUR

  Group g2  =  INTERFACES.addGroup("g2")
    .setPosition(400, 20)
    .setWidth(190)
    .activateEvent(true)
    .setBackgroundColor(color( 0xffCED2DB ))
    .setBackgroundHeight(180)
    .setLabel("COLOUR")
    .setBarHeight(14);


  //////////////////////////////////// GROUP 2 SETTINGS
  // Circles
  Slider col1 = INTERFACES.addSlider("HUE").setPosition(10, 20)
    .setRange(0, 360).setValue(230)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g2);
  Slider col2 = INTERFACES.addSlider("SAT").setPosition(10, 35)
    .setRange(0, 100).setValue(100)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g2);
  Slider col3 = INTERFACES.addSlider("BRIGHTNESS").setPosition(10, 50)
    .setRange(0, 100).setValue(100)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g2);

  // Lines

  Slider col4 = INTERFACES.addSlider("HUE2").setPosition(10, 65)
    .setRange(0, 360).setValue(0)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g2);
  Slider col5 = INTERFACES.addSlider("SAT2").setPosition(10, 80)
    .setRange(0, 100).setValue(100)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g2);
  Slider col6 = INTERFACES.addSlider("BRIGHTNESS2").setPosition(10, 95)
    .setRange(0, 100).setValue(100)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g2);


  ///////////////////////////////////////////////////////////////////// GROUP 3 PERLIN NOISE PARAMS
  Group g3  =  INTERFACES.addGroup("g3")
    .setPosition(600, 20)
    .setWidth(190)
    .close()
    .activateEvent(true)
    .setBackgroundColor(color( 0xffCED2DB ))
    .setBackgroundHeight(180)
    //.setLabel("MOTION")
    .setBarHeight(14);

  //////////////////////////////////// GROUP 3 SETTINGS
  Slider ms1 = INTERFACES.addSlider("KEN_P_AMM").setPosition(10, 20)
    .setRange(1, 100).setValue(30.0f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g3);

  //INTERFACES.getTooltip().setDelay(250);
  //INTERFACES.getTooltip().register("KEN_P_AMM", "Amount of Ken Perlin Noise");

  Slider ms2 = INTERFACES.addSlider("KEN_P_FACT").setPosition(10, 35)
    .setRange(0.1f, 1.0f).setValue(10)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g3);

  Slider ms3 = INTERFACES.addSlider("KEN_F_AMM").setPosition(10, 50)
    .setRange(0.0001f, 0.10f).setValue(0.005f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g3);
  ms3.setLabel("Noise Refine");

  mt1 = INTERFACES.addToggle("NOISE_01").setPosition(10, 70)
    .setSize(10, 10).setValue(false).setCaptionLabel("Mode1")
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g3);

  //INTERFACES.getTooltip().setDelay(250);
  //INTERFACES.getTooltip().register("NOISE_01", "2D noise with motion applied to Circles");

  mt2 = INTERFACES.addToggle("ANIMATE").setPosition(150, 70)
    .setSize(10, 10).setValue(false).setCaptionLabel("ANIME")
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g3);

  //INTERFACES.getTooltip().setDelay(250);
  //INTERFACES.getTooltip().register("ANIMATE", "Animate with mouse position");

  Toggle mt3 = INTERFACES.addToggle("NOISE_02").setPosition(60, 70)
    .setSize(10, 10).setValue(false).setCaptionLabel("Mode2")
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g3);

  //INTERFACES.getTooltip().setDelay(250);
  //INTERFACES.getTooltip().register("NOISE_02", "2D noise without motion applied to Circles");

  Toggle mt4 = INTERFACES.addToggle("ANIME_LINES").setPosition(45, 105)
    .setSize(10, 10).setValue(false).setCaptionLabel("L_ANIME")
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g3);

  Toggle mt5 = INTERFACES.addToggle("MOUSE_ANIME").setPosition(110, 70)
    .setSize(10, 10).setValue(false).setCaptionLabel("MOUSE")
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g3);

  //INTERFACES.getTooltip().setDelay(250);
  //INTERFACES.getTooltip().register("MOUSE_ANIME", "Animate with mouse pressed");

  Toggle mt6 = INTERFACES.addToggle("NOISE_THETA_LINE").setPosition(10, 105)
    .setSize(10, 10).setValue(false).setCaptionLabel("PL")
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g3);

  //INTERFACES.getTooltip().setDelay(250);
  //INTERFACES.getTooltip().register("NOISE_THETA_LINE", "Perlin Noise applied to lines & line length");

  /*
// NOT YET IMPLEMENTED
   Toggle mt7 = INTERFACES.addToggle("SUPERPERLIN_LINE").setPosition(95, 105)
   .setSize(10, 10).setValue(false).setGroup(g3).setCaptionLabel("PL")
   .setColorBackground(#F5F5F5).setColorForeground(#A09E9F)
   .setColorValueLabel(#525252).setColorActive(#F08D0C);
   mt7.setLabel("super perlin");
   */
  Slider ms4 = INTERFACES.addSlider("PL_SCALE").setPosition(10, 140)
    .setRange(0.0001f, 0.01f).setValue(0.001f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g3);

  Slider ms5 = INTERFACES.addSlider("PL_AMM").setPosition(10, 155)
    .setRange(0.0f, 250.00f).setValue(10.0f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g3);
  ms5.setLabel("PL AMM");

  //////////////////////////////////// Add GROUP 4
  Group g4  =  INTERFACES.addGroup("g4")
    .setPosition(200, 20)
    .setWidth(190)
    .activateEvent(true)
    .setBackgroundColor(color( 0xffCED2DB ))
    .setBackgroundHeight(220)
    .setLabel("GRID")
    .setBarHeight(14);



  ///////////////////////////////////////////////////////////////////// GROUP 4 SETTINGS GRID PARAMS
  Numberbox nb5 = INTERFACES.addNumberbox("STEPX").setPosition(10, 20)
    .setRange(1.0f, 60.0f).setValue(10.0f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g4);
    makeEditable( nb5 );


    Numberbox nb6 = INTERFACES.addNumberbox("STEPY").setPosition(85, 20)
    .setRange(1.0f, 60.0f).setValue(10.0f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g4);
    makeEditable( nb6 );
  //INTERFACES.getTooltip().setDelay(250);
  //INTERFACES.getTooltip().register("GRID_STEPX", "Change grid step X values");

  //INTERFACES.getTooltip().setDelay(250);
  //INTERFACES.getTooltip().register("GRID_STEPY", "Change grid step Y values");

   Numberbox nb7 = INTERFACES.addNumberbox("STEPXDOT").setPosition(10, 60)
    .setRange(1.0f, 60.0f).setValue(15)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g4);
    nb7.setLabel("STEPX_LINES");
    makeEditable( nb7 );

   Numberbox nb8 = INTERFACES.addNumberbox("STEPYDOT").setPosition(85, 60)
    .setRange(1.0f, 60.0f).setValue(15)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g4);
    nb8.setLabel("STEPY_LINES");
    makeEditable( nb8 );


  Slider gs3 = INTERFACES.addSlider("XOFF").setPosition(10, 115)
    .setRange(-50.0f, 50.0f).setValue(0.0f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g4);
  gs3.setLabel("refine x");

  Slider gs4 = INTERFACES.addSlider("YOFF").setPosition(10, 130)
    .setRange(-50.0f, 50.0f).setValue(0.0f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g4);
  gs4.setLabel("refine y");


  Bang gb1 = INTERFACES.addBang("bangSave").setPosition(10, 180)
    .setSize(15, 15).setTriggerEvent(Bang.RELEASE).setCaptionLabel("save")
    .setColorBackground(0xff0D6FBC).setColorForeground(0xff0D6FBC).setGroup(g4);
  //gb1.setLabel("save a preset");

  Bang gb2 = INTERFACES.addBang("bangLoad").setPosition(50, 180)
    .setSize(15, 15).setTriggerEvent(Bang.RELEASE).setCaptionLabel("load")
    .setColorBackground(0xff0D6FBC).setColorForeground(0xff0D6FBC).setGroup(g4);
  //gb2.setLabel("load a preset");


  Bang gb3 = INTERFACES.addBang("bangExport").setPosition(100, 180)
    .setSize(15, 15).setTriggerEvent(Bang.RELEASE).setCaptionLabel("exportPDF")
    .setColorBackground(0xff0D6FBC).setColorForeground(0xff0D6FBC).setGroup(g4);
  //gb3.setLabel("export to PDF");


  Slider gs5 = INTERFACES.addSlider("FONT_SIZE").setPosition(10, 145)
    .setRange(0.5f, 900.0f).setValue(300.0f)
    .setColorBackground(0xffF5F5F5).setColorForeground(0xffA09E9F)
    .setColorValueLabel(0xff525252).setColorActive(0xffF08D0C).setGroup(g4);
  gs5.setLabel("Font Size");


  ///////////////////////////////////////////////////////////// TEXT
  Textarea myTextarea = INTERFACES.addTextarea("txt")
    .setPosition(1020, 5)
    .setSize(240, 140)
    //.setFont(createFont("FiraSans", 12))
    .setFont(loadFont("FiraMono-Regular-12.vlw"))
    .setLineHeight(14)
    .setColor(color(128))
    .setColorBackground(color(255, 100))
    .setColorForeground(color(255, 100));
  ;
  myTextarea.setText("Alphagraph is a parametric tool for exploring letter forms."
    +"\nDeveloped by mark webster"
    +"\n"
    +"\n- Use save & load buttons for presets."
    +"\n- exportPDF button saves a vector file."
    );
}

public void controlEvent(ControlEvent theEvent) {
  if (theEvent.isGroup()) {
    println("got an event from group "
      +theEvent.getGroup().getName()
      +", isOpen? "+theEvent.getGroup().isOpen()
      );
  } else if (theEvent.isController()) {
    println("got something from a controller "
      +theEvent.getController().getName()
      );
  }
  if (theEvent.isFrom(INTERFACES.getController("FontsList"))) {
    fontIndex = (int)d1.getValue();
    getFont = s[fontIndex];
  }
}
/////////////////////////////////////////////// Event listening


/* :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: */

// Fonction pour mettre \u00e0 jour notre grille
public void updateGrid(float xOff, float yOff, float pasX, float pasY, float pasXDot, float pasYDot) {
  //updatePGraphics();
  DOT_AGENTS.renderGrid(xOff, yOff, pasX, pasY);
  LINE_AGENTS.renderGrid(xOff, yOff, pasXDot, pasYDot);
}

public void updateTextField() {
  myTextfield.setText(getFont);
}

/* :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: */

// Bang button updates
public void bangSave() {
  selectOutput("Save a preset", "fileSelectedOut");
}
public void bangLoad() {
  selectInput("Load a preset", "fileSelected");
}


public void bangExport() {
  SAVE_PDF = !SAVE_PDF;
}

// File Selection method for choosing saved presets
public void fileSelected(File selection) {
  if (selection == null) {
    println("Window was closed or the user hit cancel.");
  } else {
    println("User selected " + selection.getAbsolutePath());
    String s = selection.getAbsolutePath();
    //INTERFACES.getProperties().load(s);
    INTERFACES.loadProperties(s);
  }
}

// File saving method for saving presets
public void fileSelectedOut(File selection) {
  if (selection == null) {
    println("Window was closed or the user hit cancel.");
  } else {
    println("User selected " + selection.getAbsolutePath());
    String s = selection.getAbsolutePath();
    INTERFACES.saveProperties(s);
  }
}

//////////////////// ADDED 25 : 01 : 2017
// function that will be called when controller 'numbers' changes
public void numbers(float f) {
  println("received "+f+" from Numberbox numbers ");
}

public void makeEditable( Numberbox n ) {
  // allows the user to click a numberbox and type in a number which is confirmed with RETURN
  final NumberboxInput nin = new NumberboxInput( n ); // custom input handler for the numberbox

  // control the active-status of the input handler when releasing the mouse button inside
  // the numberbox. deactivate input handler when mouse leaves.
  n.onClick(new CallbackListener() {
    public void controlEvent(CallbackEvent theEvent) {
      nin.setActive( true );
    }
  }
  ).onLeave(new CallbackListener() {
    public void controlEvent(CallbackEvent theEvent) {
      nin.setActive( false );
      nin.submit();
    }
  }
  );
}



// input handler for a Numberbox that allows the user to
// key in numbers with the keyboard to change the value of the numberbox

public class NumberboxInput {

  String text = "";
  Numberbox n;
  boolean active;

  NumberboxInput(Numberbox theNumberbox) {
    n = theNumberbox;
    registerMethod("keyEvent", this );
  }

  public void keyEvent(KeyEvent k) {
    // only process key event if input is active
    if (k.getAction()==KeyEvent.PRESS && active) {
      if (k.getKey()=='\n') { // confirm input with enter
        submit();
        return;
      } else if (k.getKeyCode()==BACKSPACE) {
        text = text.isEmpty() ? "":text.substring(0, text.length()-1);
        //text = ""; // clear all text with backspace
      } else if (k.getKey()<255) {
        // check if the input is a valid (decimal) number
        final String regex = "\\d+([.]\\d{0,2})?";
        String s = text + k.getKey();
        if ( java.util.regex.Pattern.matches(regex, s ) ) {
          text += k.getKey();
        }
      }
      n.getValueLabel().setText(this.text);
    }
  }

  public void setActive(boolean b) {
    active = b;
    if (active) {
      n.getValueLabel().setText("");
      text = "";
    }
  }

  public void submit() {
    if (!text.isEmpty()) {
      n.setValue( PApplet.parseFloat( text ) );
      text = "";
    } else {
      n.getValueLabel().setText(""+n.getValue());
    }
  }
}
  public void settings() {  size(1280, 720, JAVA2D);  smooth(); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "--present", "--window-color=#666666", "--hide-stop", "alphagraph_09" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
